import json
import requests
import urllib.request
import gzip
from bs4 import BeautifulSoup
class GEOMinner:
    def __init__(self,geoID):
        self.geoId=geoID
        self.series,self.platform,self.samples=self.getMetadataFromGEOID()

    def getDownloadLinks(self):
        url="https://www.ncbi.nlm.nih.gov/gds/?term="+str(self.geoId)+"&report=DocSums&format=text"
        GeoIdnnn=self.geoId[:len(self.geoId)-3]+"nnn"
        downloadLinks='geo/series/'+GeoIdnnn+'/'+self.geoId+'/'
        return downloadLinks

    def getSeriesMetadata(self,seriesXML):

        series_status={}
        for series_stat in seriesXML.status.children:
            series_status[series_stat.name]=series_stat.string

        series_info={}
        for sample in seriesXML.children:
            series_info[sample.name]=sample.string

        series_info.update(series_status)

        return series_info


    def getPlatformMetadata(self,platformXML):

        platform_status={}
        for platform_stat in platformXML.status.children:
            platform_status[platform_stat.name]=platform_stat.string

        platform_info={}
        for platform in platformXML.children:
            platform_info[platform.name]=platform.string

        platform_info.update(platform_status)
        #print(platform_info)
        return platform_info

    def getSampleMetadata(self,sample):
        sample_status={}
        for samp_stat in sample.status.children:
            sample_status[samp_stat.name]=samp_stat.string

        channel={}
        for chan_stat in sample.channel.children:
            channel[chan_stat.name]=chan_stat.string

        channel_char={}
        for chan_stat_char in sample.channel.find_all("characteristics"):
            channel[str(chan_stat_char.attrs["tag"])]=chan_stat_char.string

        sample_info={}
        for sample in sample.children:
            sample_info[sample.name]=sample.string


        platforms={}
        for platform in sample.find_all("platform-ref"):
            for j in platform:
                print(j.name)
                platforms[j.name]=j.string
        channel.update(channel_char)
        sample_info.update(sample_status)
        sample_info.update(channel)
        sample_info.update(platforms)

        return sample_info

    def getMetadataFromGEOID(self):

        url="https://ftp.ncbi.nlm.nih.gov/"+self.getDownloadLinks()+"miniml/"+self.geoId+"_"+"family.xml.tgz"
        r=urllib.request.urlopen(url)
        rd = gzip.decompress(r.read())
        data=BeautifulSoup(rd,"html5lib")
        #print(data.sample.channel.find("characteristics").attrs["tag"])
        samples={}
        for sample in data.find_all("sample"):#.children:
            #print(sample)
            samples[sample.get("iid")] =self.getSampleMetadata(sample)
            samples[sample.get("iid")]["series_accsesion"]=self.geoId

            for i in sample.find_all("relation"):
                if i.get("type")=="BioSample":
                    #print(i.get("target")+"?report=full&format=text")
                    samples[sample.get("iid")]["biosampleLink"]=i.get("target")+"?report=full&format=text"
                elif i.get("type")=="SRA":
                    samples[sample.get("iid")]["SRALink"]=i.get("target")+"&report=FullXml"


        series={}
        for serie in data.find_all("series"):#.children:
            #series[serie.get("iid")] =self.getSeriesMetadata(serie)
            series =self.getSeriesMetadata(serie)
            #series[sample.get("iid")]["series_accsesion"]=GEOID
        platforms={}
        for platform in data.find_all("platform"):#.children:
            #platforms.self.getPlatformMetadata(platform)
            if platform.get("iid") not in list(platforms.keys()):
                platforms[platform.get("iid")]=(self.getPlatformMetadata(platform))

        print(platforms)

        return series, platforms, samples
    def getSamples(self):
        return self.samples
    def getPlatform(self):
        return self.platform
    def getSeries(self):
        return self.series



import seaborn as sns; sns.set(color_codes=True)
import numpy as np
import matplotlib.pyplot as plt
def heatmap(data, cluster_marker=None, color_label=None, row_names=None, col_names=None, row_color=None, col_color=None, row_cluster=False, col_cluster=False, cbar_kws=None):
        
    g = sns.clustermap(data,row_colors=row_color,col_colors=col_color, row_cluster=False, col_cluster=False, z_score=0, cmap="mako", robust=True, figsize=(8,8))   
    
    #color_labell=dict(sorted(color_label.items()))
    for label in sorted(color_label.keys()):
        g.ax_col_dendrogram.bar(0, 0, color=color_label[label],
                            label=label, linewidth=0)
    
    g.ax_col_dendrogram.legend(loc="center", ncol=6)
    
    g.cax.set_position([.15, .2, .03, .45])
    
    return g








import numpy as np 
import pandas as pa
from scipy.sparse import csr_matrix, isspmatrix
import pyensembl 
from pyensembl import EnsemblRelease

def normalizeUMIWithscalefactor(data, scale_factor=10e6):
    cellNormalizedData=np.log(1+(data/np.sum(data,axis=1)[0]))*scale_factor
    return(cellNormalizedData)

def gene_Length(list_of_genes, ensembl_id=True, species="human", Ensembl_Release=75):
    gn=EnsemblRelease(species=species, release=Ensembl_Release)
    if ensembl_id:
        gene_pos=list(map(gn.locus_of_gene_id, list_of_genes))
        gene_pos_end=[i.end for i in gene_pos]
        gene_pos_start=[i.start for i in gene_pos]
        
    else:
        gene_pos=list(map(gn.loci_of_gene_names, list_of_genes))
        gene_pos_end=[i[0].end for i in gene_pos]
        gene_pos_start=[i[0].start for i in gene_pos]

    gene_len=np.array(gene_pos_end)-np.array(gene_pos_start)
    return pa.DataFrame({'gene_names':list_of_genes, "gene_length":gene_len/1000})    

def getTPM(rowCountData, gene_Names=None, index_column=None, ensembol_gene=False, species="human", Ensembl_release=75):

    if(isspmatrix(rowCountData)):
        rowCountData=pa.DataFrame(rowCountData.toarray())
        rowCountData.index=gene_Names
        
    else:
        if index_column is not None:
            rowCountData.index=rowCountData[index_column]
            rowCountData=rowCountData.drop([index_column], axis=1)

    if ensembol_gene:
        known_genes=list(map(pyensembl.common.is_valid_ensembl_id,gene_Names)) 
        rowCountData=rowCountData.iloc[known_genes,:]
        
        rowCountData.index=np.array(gene_Names)[np.array(known_genes)]
        
        gene_length=gene_Length(gene_Names, ensembl_id=ensembol_gene, species=species, Ensembl_Release=Ensembl_release) 
    else:

        gene_length=gene_Length(gene_Names, ensembl_id=ensembol_gene, species=species, Ensembl_Release=Ensembl_release) 
        
    if(gene_length.shape[0]==rowCountData.shape[0]):
        
        count_data_RKB=rowCountData.div(list(gene_length["gene_length"]), axis=0)
    
        count_data_TPM=np.array(count_data_RKB)/ np.array(rowCountData.sum(axis=0)).reshape((1,len(rowCountData.sum(axis=0))))
    
        count_data_TPM=pa.DataFrame(count_data_TPM*10e6)
        count_data_TPM.index=list(rowCountData.index)
        count_data_TPM.columns=list(rowCountData.columns)
    
        count_data_TPM=count_data_TPM[(count_data_TPM.T != 0).any()].dropna(axis=0)
    else:
        print("one or more gene ids are not annoatated under", Ensembl_Release, "Please use different release")
        
    return count_data_TPM.T, list(count_data_TPM.index)

def ENSEMBLID_to_geneSymbol(ENSEMBL, Ensembl_Release=75):        
    data=EnsemblRelease(Ensembl_Release)
    if type(ENSEMBL) is list:
        Genes=list(map(data.gene_name_of_gene_id,ENSEMBL))
    else:
        Genes=data.gene_name_of_gene_id(ENSEMBL)
    return Genes

